///
/// \file NumberCrush.cxx
/// \brief code c++ NumberCrush
/// \author GAMELIA, GANGUET, GONZALEZ, JUY
/// \version 1.3
/// \date 12/01/2015
/// Projet code c++ for the game: "NumberCrush" 
///


#include <iostream>

#include <vector>

#include <stdio.h> 

#include <stdlib.h>     

#include <time.h> 

#include <utility>

#include <iomanip>

using namespace std;


///
/// \namespace std
/// Anonymous namespace
///


namespace

{

///
/// \CVLine
/// \brief A type representing a row on the grid

/// \CMat
/// \brief A type representing the line

/// \CPosition 
/// \brief A position in the grid
///


	typedef vector <unsigned> CVLine; 

	typedef vector <CVLine> CMat; 

	typedef pair <unsigned, unsigned> CPosition; 

///
/// \brief Clear the screen 
/// Allows you to clear the screen
///


	void ClearScreen ()

	{

    	cout << "\033[H\033[2J";

	}

	const string KReset   ("0");

	const string KNoir    ("30");

	const string KRouge   ("31");

	const string KVert    ("32");

	const string KJaune   ("33");

	const string KBleu    ("34");

	const string KMAgenta ("35");

	const string KCyan    ("36");

///
/// \brief Change the color
/// Allows you to change the color of the grid's digits
///


	void Couleur (const string & coul)

	{

    	cout << "\033[" << coul <<"m";

	}

///
/// \brief initializing of the grid
/// Enables initialisation of your grid before playing
///


	void InitGrid (CMat & Grid, unsigned Size)

	{

		srand (time(NULL));

		unsigned ValInput;

		const unsigned KNbCandies = 6;

		Grid.resize(Size + 1);

		for(unsigned i = 0; i<Grid.size(); ++i)

		{	

			Grid[i].resize(Size + 1);

			for(unsigned j =0; j<Grid[i].size(); ++j)

			{

				ValInput = rand() % KNbCandies + 1;					

				Grid[i][j] = ValInput;

				if(i == 0)

					Grid[i][j] = j;

				if(j == 0)

					Grid[i][j] = i;						

			}

		}

			

	}


///
/// \brief Displays the grid
/// Displays the grid of the game
///


	void  DisplayGrid (const CMat & Grid, unsigned ActualScore)

	{

		ClearScreen();



		for(unsigned i = 0; i<Grid.size(); ++i)

		{

			for( unsigned j = 0; j<Grid[i].size(); ++j)

			{

				if( Grid[i][j] == 0)

				{

					cout  << setw(4) << "" ;

					Couleur (KReset);

				}

				else

				{			

					switch(Grid[i][j])

					{

						case(1):

						{

							Couleur(KRouge);

							cout << setw(4) << Grid[i][j];

							Couleur (KReset);

							break;						

						}	

						case(2):

						{

							Couleur(KBleu);

							cout << setw(4) << Grid[i][j];

							Couleur (KReset);

							break;						

						}

						case(3):

						{

							Couleur(KVert);

							cout << setw(4) << Grid[i][j];

							Couleur (KReset);

							break;						

						}

							case(4):

						{
							
							Couleur(KJaune);
							
							cout << setw(4) << Grid[i][j];
							
							Couleur (KReset);
						
						}
						
						case(5):
						
						{
						
							Couleur(KCyan);
							
							cout << setw(4) << Grid[i][j];
							
							Couleur (KReset);
						
						}
						
						case(6):
						
						{
							
							Couleur(KMAgenta);
							
							cout << setw(4) << Grid[i][j];
							
							Couleur (KReset);
						
							}

						default:

						{

							cout << setw(4) << Grid[i][j];

							Couleur (KReset);

							break;

						}

					}

	

				}

			}

			cout << endl;

		}

		cout << "Score : " << setw(4) << ActualScore << endl;

	}

	

}


/// 
/// \brief Verification of 3 identicals values on the same line
/// Method that allows to know the number of result of 3 identical figures in one line of the grid
/// \param AtLeastThreeInARow : Following of three identical numbers in a line of the grid
/// \return true if the grid has a series of 3 or more digits in a line of the grid,
/// false otherwise
///


bool AtLeastThreeInARow (const CMat & Grid, CPosition & Pos, unsigned & HowMany)

{

/// 
/// \brief Checking the precense of 3 identical digits
/// Method that allows to know the number of result of 3 identical figures in the grid
/// \param ThreeOrMore : Following of three identical numbers in the grid
/// \return true if the grid has series of 3 or more digits in the grid,
/// false otherwise
/// 


	bool ThreeOrMore = false;

	unsigned x = Pos.first;

	unsigned y = Pos.second;

	unsigned ValeurTemoin = Grid[x][y];

	HowMany = 1;

	while(y < (Grid[x].size() - 1) && ValeurTemoin == Grid[x][y + 1])

	{

		y += 1;

		HowMany += 1;

	}

	y = Pos.second;

	while(y > 1 && ValeurTemoin == Grid[x][y - 1])

	{

		y -= 1;

		HowMany += 1;

	}

	if(HowMany >= 3)

		ThreeOrMore = true; 

	cout << "Lign" << HowMany << setw(4) << ThreeOrMore << endl;

	return ThreeOrMore;

}

/// 
/// \brief Verification of 3 identicals values on the same column
/// Method that allows to know the number of result of 3 identical figures in one column of the grid
/// \param AtLeastThreeInAColumn : Following of three identical numbers in column of the grid
/// \return true if the grid has a series of 3 or more digits in a column of the grid,
/// false otherwise
///


bool AtLeastThreeInAColumn (const CMat & Grid, CPosition & Pos, unsigned & HowMany)

{

/// 
/// \brief Checking the precense of 3 identical digits
/// Method that allows to know the number of result of 3 identical figures in the grid
/// \param ThreeOrMore : Following of three identical numbers in the grid
/// \return true if the grid has series of 3 or more digits in the grid,
/// false otherwise
/// 


	bool ThreeOrMore = false;

	unsigned x = Pos.first;

	unsigned y = Pos.second;

	unsigned ValeurTemoin = Grid[x][y];

	HowMany = 1;

	while(x < (Grid.size() - 1) && ValeurTemoin == Grid[x + 1][y])

	{

		x += 1;

		HowMany += 1;

	}

	x = Pos.first;

	while(x > 1 && ValeurTemoin == Grid[x - 1][y])

	{

		x -= 1;

		HowMany += 1;

	}

	if(HowMany >= 3)

		ThreeOrMore = true; 

	cout << "Column" << HowMany << setw(4) << ThreeOrMore << endl;

	return ThreeOrMore;

}


///
/// \brief Displays the score
/// Displays the score of your current game
///


void Score(unsigned & ActualScore, unsigned & HowMany)

{

	unsigned CopyHowMany = HowMany;
	do
	{
		ActualScore = ActualScore + CopyHowMany;
		CopyHowMany = CopyHowMany - 1;
	}while(CopyHowMany != 0);

}



///
/// \brief Remove a column
/// Remove a column after the player did a following of 3 same digits in a column of the grid
///

void RemovalInColumn (CMat & Grid, const CPosition & Pos, unsigned  HowMany)

{ 

	const unsigned KImpossible = 0;

	unsigned ValeurPremierX;

	unsigned x = Pos.first;

	unsigned y = Pos.second;

	unsigned TestValeur = Grid[x][y];

	while(x > 1 && TestValeur == Grid[x - 1][y])

	{

		x = x - 1;

	}

	ValeurPremierX = x;

	while(x < ValeurPremierX + HowMany)

	{

		Grid[x][y] = Grid[x + HowMany][y];

		x = x + 1;

	}

	x = Grid.size() - 1 - HowMany;

	while(x < Grid.size())

	{

		Grid[x][y] = KImpossible;

		x = x + 1;

	}

	

}



///
/// \brief Remove a line
/// Remove a line after the player did a following of 3 same digits in a line of the grid
///


void RemovalInRow (CMat & Grid, const CPosition & Pos, unsigned  HowMany)

{
	const unsigned KImpossible = 0;

	unsigned ValeurPremierY;

	unsigned x = Pos.first;

	unsigned y = Pos.second;

	unsigned TestValeur = Grid[x][y];

	while(y > 1 && TestValeur == Grid[x][y - 1])

	{

		y = y - 1;

	}

	ValeurPremierY = y;

	while(y < ValeurPremierY + HowMany)

	{

		Grid[x][y] = Grid[x][y + 1];

		y = y + 1;

	}

	y = ValeurPremierY;

	while(HowMany >  0)

	{

		Grid[Grid.size()-1][y] = KImpossible;

		y = y + 1;

	}

	

}

///
/// \brief Make move a digit of the grid
/// Allows you to move one digit of the grid with one of these adjacent digits
///


void MakeAMove (CMat & Grid, const CPosition & Pos, char Direction)

	{

		switch (Direction)

		{

		

			case ('d'):

			{



				if (Pos.second+1 < Grid.size() )

				swap(Grid[Pos.first][Pos.second],Grid[Pos.first][Pos.second+1]); //Pos.fisrt = ligne et pos.second=colonne (pos.first+1=from top... pos.second+1=from left)

				break;		

			}	

			case('q'):

			{

				if (Pos.second-1 >= 0)

				swap(Grid[Pos.first][Pos.second],Grid[Pos.first][Pos.second-1]);

				break;		

			}

			case('z'):

			{

			

				if (Pos.first-1 >= 0)

				swap(Grid[Pos.first][Pos.second],Grid[Pos.first-1][Pos.second]);

				break;

			}

			case('s'):

			{

				if (Pos.first+1 < Grid.size())

				swap(Grid[Pos.first][Pos.second],Grid[Pos.first+1][Pos.second]);

				break;		

			}

		}

	}



///
/// \fn int main (void)
/// \brief Program entry
/// \return 0 
///


int main ()

{	

	cout << "Welcome to our NumberCrush"<<endl;

	cout << "Select the abscissa of the number that you want to move (it starts at 1)"<<endl;

	cout << "Select ordered of this number"<<endl;

	cout << "Then select a movement"<<endl;

	cout << "UP : z"<<endl;

	cout << "DOWN : s"<<endl;

	cout << "LEFT : q"<<endl;

	cout << "RIGHT : d"<<endl;



	bool Sucess = false;

	unsigned ActualScore = 0;

	unsigned HowMany = 0;

	vector<vector<unsigned> > Grid;

	cout << "What size array?" << endl;

	unsigned Size;

	cin >> Size;

	pair <unsigned, unsigned> Pos;

	unsigned ValPos;

	ValPos = Size/2;

	Pos = std::make_pair (ValPos,ValPos);

	InitGrid(Grid,Size);

	DisplayGrid(Grid, ActualScore);

	char Direction;

	while(Direction != 'p')

	{

		cin >> Pos.second;

		cin >> Pos.first;

		cin >> Direction;

		MakeAMove(Grid,Pos,Direction);

		switch(Direction)

		{

			case('z'):

			{

				Pos.first -= 1;

				break;

			}

			case('q'):

			{

				Pos.second -= 1;

				break; 

			}

			case('s'):

			{

				Pos.first += 1;

				break;

			}

			case('d'):

			{

				Pos.second += 1;

				break;

			}

			default:

			{

				break;

			}

		}

		Sucess = AtLeastThreeInARow(Grid,Pos,HowMany);

		if(Sucess)

		{

			Score(ActualScore, HowMany);

			RemovalInRow(Grid,Pos,HowMany);

		}

		Sucess = false;

		Sucess = AtLeastThreeInAColumn (Grid, Pos, HowMany);

		if(Sucess)
		{

			Score(ActualScore, HowMany);
			
			RemovalInColumn(Grid,Pos,HowMany);

		}
		
		Sucess = false;

		DisplayGrid(Grid, ActualScore);

	}

	return 0;

} 
